#include "TensorGlyphRenderer.h"
#include <GLUT/glut.h>									//GLUT library
#include <iostream>


using namespace std;


TensorGlyphRenderer::TensorGlyphRenderer(): resolution(16),scale(1)
{  }


TensorGlyphRenderer::~TensorGlyphRenderer()
{  }


void TensorGlyphRenderer::draw(Volume<Point3d>* eigenvectors[])
{  	
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);	
	glShadeModel(GL_SMOOTH);
	glEnable(GL_COLOR_MATERIAL);
	
	glMatrixMode(GL_MODELVIEW);											//We will next modify the modelview matrix, so save it

	const Point3d x_axis(1,0,0),y_axis(0,1,0);

	for(int i=0,N=seeds.size();i<N && i<max_seeds;++i)					//Construct and draw glyphs at all seed points:
	{
		const Point3d& seed = seeds[i];									//Get current seed point
		
		Point3d e0 = (*eigenvectors[0])(seed.x,seed.y,seed.z);	//Get values of the three eigenvectors at that point
		Point3d e1 = (*eigenvectors[1])(seed.x,seed.y,seed.z);
		Point3d e2 = (*eigenvectors[2])(seed.x,seed.y,seed.z);
		float e0length = e0.norm();										//Get lengths of the three eigenvectors
		float e1length = e1.norm();
		float e2length = e2.norm();
		e0 /= e0length;													//Normalize the eigenvectors	
		e1 /= e1length;													//(needed for easier rotation math below)
		e2 /= e2length;


		glPushMatrix();													//1. Save the modelview matrix, since we're going to change it
	
		glTranslatef(seed.x,seed.y,seed.z);								//1. Translate the rotated and scaled glyph to its seed point	
		
		Point3d axis1,axis2; float angle1,angle2;
				
		axis1 = x_axis.cross(e0);										//2.2 Rotate so that x axis (of the glyph) matches eigenvector e0.
		bool rotation_1 = axis1.norm()>1.0e-4;							//    This means rotating along axis (1,0,0) x e0, with an angle
																		//    equal to (1,0,0) * e0. We only do this if e0 is not already aligned
		if (rotation_1)													//    with the x axis.
		{																//2.3 Next, rotate so that the y axis (of the glyph, following the
			angle1  = acos(e0.x);										//    above rotation) matches eigenvector e1.	
																		//    For this, we first compute axis2 = glyph's y-axis rotated with	
																		//    angle1 around axis1. Next, we rotate (around e0) so as to																																							
			float s = sin(angle1);										//    align axis2 with e1, i.e. rotate with an angle		
			float c = cos(angle1);										//    angle2 = axis2 * e1.	
			axis2   = Point3d(axis1.x*axis1.y*(1-c)-axis1.z*s,				
							  axis1.y*axis1.y*(1-c)+c,
							  axis1.y*axis1.z*(1-c)+axis1.x*s);
		}
		else axis2 = y_axis;											//   If the x-axis==e0, then the second rotation simply aligns
																		//   the y-axis with e1.

		angle2 = acos(axis2.dot(e1));
			
		glRotatef(180*angle2/M_PI,e0.x,e0.y,e0.z);						//Take care, glRotate() wants rotation angles in degrees	
				
		if (rotation_1) glRotatef(180*angle1/M_PI,axis1.x,axis1.y,axis1.z);				
		
		glScalef(e0length,e1length,e2length);							//2.4 Scale the glyph along x,y,z using eigenvalues
																		//    (i.e., norms of eigenvectors e0,e1,e2)


		float R,G,B;													//3. Compute color that reflects the direction
		R = fabs(e0.x);													//   of major eigenvector
		G = fabs(e0.y);
		B = fabs(e0.z);
		glColor3f(R,G,B);

	
		glutSolidSphere(scale,resolution,resolution);					//4. Draw glyph as a GLUT solid, shaded, sphere
																		//   The previous scalings and rotations create the desired ellipsoid
		
		glPopMatrix();													//5. Done with rotating the current glyph, so restore the modelview matrix
	}
}


