#pragma once

#include "Volume.h"
#include "Point3d.h"
#include <vector>





class TensorGlyphRenderer								//Renders 3D ellipsoid glyphs scaled and oriented along the eigenvectors of
{														//a tensor field, and placed at a user-provided set of seed points.
public:													//Glyphs are color-coded by the direction of the major eigenvector.

				TensorGlyphRenderer();
			   ~TensorGlyphRenderer();

void			setSeeds(const std::vector<Point3d>& s)
				{ seeds = s; }							//Pass the seed-set where we want to draw glyphs
				
void			setMaxSeeds(int s)
				{ 
				  if (s<0) s=0;
				  if (s>seeds.size()) s=seeds.size();
				  max_seeds = s; 
				}
				
int				getMaxSeeds()
				{ return max_seeds; }	

void			setScale(float s)
				{ scale = s; }
				
float			getScale()
				{ return scale; }

void			draw(Volume<Point3d>** eigenvectors);	//Draw glyphs in a given eigenvector field. The field is passed as:
														// eigenvectors[0]: major eigenvector
														// eigenvectors[1]: medium eigenvector
														// eigenvectors[2]: minor eigenvector
														//All eigenvectors should be scaled by their corresponding eigenvalues 

protected:

std::vector<Point3d> seeds;
int				     max_seeds;
float				 scale;
int					 resolution;
};




						

